/**********************************************************************
 * CD Jukebox by Nathan Strong (gblues@jps.net)
 *
 * This software is covered by the GNU Public License. See COPYING
 * file for details on the license. All code contained in these files
 * is (c) 1999 Nathan Strong.
 **********************************************************************/

#include "main.h"

static struct termios initial_settings, new_settings;

long current_time;

void update_clock(int secs);

void init_kb(void)
{
  tcgetattr(0, &initial_settings);
  new_settings = initial_settings;
  new_settings.c_lflag    &= ~ICANON;
  new_settings.c_lflag    &= ~ECHO;
  new_settings.c_lflag    &= ~ISIG;
  new_settings.c_cc[VMIN]  = 1;
  new_settings.c_cc[VTIME] = 0;
  tcsetattr(0, TCSANOW, &new_settings);
}

unsigned char kbhit(void)
{
  unsigned char ch;
  int nread;

  new_settings.c_cc[VMIN] = 0;
  tcsetattr(0, TCSANOW, &new_settings);
  nread = read(0, &ch, 1);
  new_settings.c_cc[VMIN] = 1;
  tcsetattr(0, TCSANOW, &new_settings);

  return nread ? ch : 0;
}

void purge_input(void)
{
  char ch;

  new_settings.c_cc[VMIN] = 0;
  tcsetattr(0, TCSANOW, &new_settings);
  while( read(0, &ch, 1) );
  new_settings.c_cc[VMIN] = 1;
  tcsetattr(0, TCSANOW, &new_settings);
}

void close_kb(void)
{
  tcsetattr(0, TCSANOW, &initial_settings);
}

int main(int argc, char **argv)
{
  struct timeval cur_time;

  gettimeofday(&cur_time, NULL);
  current_time = (long) cur_time.tv_sec;
  init_kb();
#ifdef DEBUG
  setcorelimit();
#endif
  if( ! getuid() )
  {
    create_config();
  }
  printf("Please wait while drives are scanned.\n");
  load_tochdrs();
  boot_ui();
  close_kb();
  exit(0);
}

void program_loop(void)
{
  struct timeval last_time;

  gettimeofday(&last_time, NULL);

  for( ; ; )
  {
    unsigned char c;
    c = kbhit();
    switch(c)
    {
      case 'q': case 'Q':
        return;
        break;
      case 'j': case 'J': case 'i': case 'I':
        move_cursor_left();
        break;
      case 'l': case 'L': case 'k': case 'K':
        move_cursor_right();
        break;
      case '\033':
      {
        unsigned char ch;
        kbhit();
        ch = kbhit();
        switch(ch)
        {
          case 'A': case 'D':
            move_cursor_left();
            break;
          case 'B': case 'C':
            move_cursor_right();
            break;
        }
        break;
      }
      case '\n':
      {
        int i;
        for( i = 0; ui.commands[i].name[0] != '\0'; i++ )
        {
          if( ui.commands[i].selected )
          {
            ui.commands[i].fun();
            break;
          }
        }
        break;
      }
    }
    {
      struct timeval now_time;
      long usecDelta;
      long secDelta;
      gettimeofday(&now_time, NULL);
      usecDelta = ((int)last_time.tv_usec) - ((int)now_time.tv_usec)
      + 1000000 / 16;
      secDelta  = ((int)last_time.tv_sec)  - ((int)now_time.tv_sec);
      while( usecDelta < 0 )
      {
        usecDelta += 1000000;
        secDelta--;
      }
      while( usecDelta > 1000000 )
      {
        usecDelta -= 1000000;
        secDelta++;
      }
      if( secDelta > 0 || (secDelta == 0 && usecDelta > 0) )
      {
        struct timeval stall_time;

        stall_time.tv_usec = usecDelta;
        stall_time.tv_sec  = secDelta;
        if( select(0, NULL, NULL, NULL, &stall_time) < 0 )
        {
          perror("program_loop: select: stall");
          exit_cdplayer();
        }
      }
    }
    gettimeofday(&last_time, NULL);
    if( last_time.tv_sec > current_time && ui.playing == PLAYING)
    {
      update_clock((int) last_time.tv_sec);
    }
    if( ui.playing == PAUSED ) current_time = (int) last_time.tv_sec;
  }
}

void move_cursor_right(void)
{
  int i;

  for(i = 0; ui.commands[i].name[0] != '\0'; i++)
  {
    if( ui.commands[i].selected )
    {
      if( ui.commands[i+1].name[0] == '\0' ) return;
      ui.commands[i].selected = FALSE;
      ui.commands[i+1].selected = TRUE;
      SET_BIT(ui.dirty, DIRTY_UI);
      update_ui();
      break;
    }
  }
}

void move_cursor_left(void)
{
  int i;

  if( ui.commands[0].selected ) return;

  for( i = 0; ui.commands[i].name[0] != '\0'; i++ )
  {
    if( ui.commands[i].selected )
    {
      ui.commands[i].selected = FALSE;
      ui.commands[i-1].selected = TRUE;
      SET_BIT(ui.dirty, DIRTY_UI);
      update_ui();
      break;
    }
  }
}

void exit_cdplayer(void)
{
  close_kb();
  exit(0);
}

#ifdef DEBUG
void setcorelimit(void)
{
  struct rlimit r;

  r.rlim_cur = r.rlim_max = RLIM_INFINITY;
  setrlimit(RLIMIT_CORE, &r);
}
#endif

void update_clock(int newtime)
{
  int len, pos;
  int diff;
  diff = newtime - current_time;
  current_time = newtime;
  ui.cur_sec = ui.cur_sec + diff;
  if( ui.cur_sec >= 60 )
  {
    ui.cur_min++;
    ui.cur_sec  = ui.cur_sec % 60;
  }
  len = ui.cur_track->length;
  pos = (ui.cur_min * 60) + ui.cur_sec;
  if( pos > len )
  {
    pos -= len;
    SET_BIT(ui.dirty, DIRTY_TRACK);
    ui.cur_min = pos / 60;
    ui.cur_sec = pos % 60;
    if( ui.cur_track->next_playlist )
      do_next_track();
    else
      stop_playing();
  }

  SET_BIT(ui.dirty, DIRTY_CLOCK);
  update_ui();
}

CD_TRACK *generate_playlist(bool random)
{
  DRIVE_DATA *dr;
  CD_TRACK *trk, *list = NULL, *newlist = NULL, *start;

  for( dr = config.drives; dr != NULL; dr = dr->next )
  {
    if( dr->toc == NULL ) continue;
    for( trk = dr->tracks; trk != NULL; trk = trk->next )
    {
      trk->next_playlist = list;
      trk->prev_playlist = NULL;
      if( list )
        list->prev_playlist = trk;
      list = trk;
    }
  }
  while( (start = list) != NULL )
  {
    list = list->next_playlist;
    start->next_playlist = newlist;
    if( newlist ) newlist->prev_playlist = start;
    newlist = start;
  }
  list = newlist;
  return list;
}

